import { AnyJSON, Diff, JSONObject, Omit, Overwrite } from './util';
export declare type SchemaUpdate<State extends SchemaState, K extends keyof SchemaState, V extends SchemaState[K]> = Schema<Omit<State, K> & {
    [P in K]: V;
}>;
export declare type SchemaMultiUpdate<State extends SchemaState, K extends keyof SchemaState, U extends {
    [P in K]: SchemaState[P];
}> = Schema<Overwrite<State, U>>;
export declare type TypeNames = 'string' | 'number' | 'integer' | 'boolean' | 'null' | 'array' | 'object';
export interface TypeDefs<State extends SchemaState> {
    any: any;
    string: string;
    number: number;
    integer: number;
    boolean: boolean;
    null: null;
    array: Array<State['items'] | State['additionalItems']>;
    object: State['anyOf'] & State['oneOf'] & State['allOf'] & {
        [P in State['required']]: State['properties'][P];
    } & {
        [P in Diff<keyof State['properties'], State['required']>]?: State['properties'][P];
    } & {
        [P in State['additionalProperties']['key'] | State['patternProperties']['key']]?: State['additionalProperties']['type'] | State['patternProperties']['type'];
    };
    enum: State['enum'];
    const: State['const'];
}
export interface SchemaState {
    type: keyof TypeDefs<DefaultSchemaState>;
    items: any;
    additionalItems: any;
    properties: any;
    required: string;
    additionalProperties: {
        key: string;
        type: any;
    };
    patternProperties: {
        key: string;
        type: any;
    };
    enum: any;
    anyOf: any;
    oneOf: any;
    allOf: any;
    const: any;
}
export interface DefaultSchemaState {
    type: 'any';
    items: any;
    additionalItems: never;
    properties: any;
    required: never;
    additionalProperties: {
        key: string;
        type: any;
    };
    patternProperties: {
        key: never;
        type: never;
    };
    enum: any;
    const: any;
    anyOf: {};
    oneOf: {};
    allOf: {};
}
export declare class Schema<State extends SchemaState = DefaultSchemaState> {
    readonly props: JSONObject;
    private readonly state;
    TypeOf: TypeDefs<State>[State['type']];
    constructor(props?: JSONObject, state?: JSONObject);
    setProps(props: JSONObject): any;
    getState(key: string): AnyJSON;
    setState(key: string, val: AnyJSON): void;
    toJSON(): JSONObject;
    type<TypeKeys extends TypeNames>(type: TypeKeys[] | TypeKeys): SchemaUpdate<State, 'type', TypeKeys>;
    enum<T extends AnyJSON>(values: T[]): SchemaMultiUpdate<State, 'enum' | 'type', {
        type: 'enum';
        enum: T;
    }>;
    const<T extends AnyJSON>(constValue: T): SchemaMultiUpdate<State, 'const' | 'type', {
        type: 'const';
        const: T;
    }>;
    anyOf<S extends Schema<any>>(schemas: S[]): SchemaUpdate<State, 'anyOf', S['TypeOf']>;
    oneOf<S extends Schema<any>>(schemas: S[]): SchemaUpdate<State, 'oneOf', S['TypeOf']>;
    allOf<S extends Schema<any>>(schemas: S[]): SchemaUpdate<State, 'allOf', S['TypeOf']>;
    title(title: string): this;
    description(description: string): this;
    default(defaultValue: AnyJSON): this;
    example<S extends Schema<any>>(example: S['TypeOf']): this;
    not<X extends SchemaState>(schema: Schema<X>): this;
    items<ItemsSchema extends Schema<any>>(items: ItemsSchema[] | ItemsSchema): SchemaUpdate<State, 'items', ItemsSchema['TypeOf']>;
    additionalItems(additionalItems: true): SchemaUpdate<State, 'additionalItems', any>;
    additionalItems(additionalItems: false): this;
    additionalItems<ItemsSchema extends Schema<any>>(additionalItems: ItemsSchema): SchemaUpdate<State, 'additionalItems', ItemsSchema['TypeOf']>;
    contains(contains: Schema<any>): this;
    maxItems(maxItems: number): this;
    minItems(minItems: number): this;
    uniqueItems(uniqueItems: boolean): this;
    properties<Properties extends {
        [P: string]: Schema<any>;
    }>(properties: Properties): SchemaUpdate<State, 'properties', {
        [K in keyof Properties]: Properties[K]['TypeOf'];
    }>;
    required<K extends string>(...required: K[]): SchemaUpdate<State, 'required', K>;
    additionalProperties(additionalProperties: true): SchemaUpdate<State, 'additionalProperties', {
        key: string;
        type: any;
    }>;
    additionalProperties(additionalProperties: false): SchemaUpdate<State, 'additionalProperties', {
        key: never;
        type: never;
    }>;
    additionalProperties<T extends Schema<any>>(additionalProperties: T): SchemaUpdate<State, 'additionalProperties', {
        key: string;
        type: T['TypeOf'];
    }>;
    patternProperties<T extends Schema<any>>(patternProperties: {
        [key: string]: T;
    }): SchemaUpdate<State, 'patternProperties', {
        key: string;
        type: T['TypeOf'];
    }>;
    dependencies(dependencies: {
        [key: string]: Schema<any> | string[];
    }): this;
    maxProperties(maxProperties: number): this;
    minProperties(minProperties: number): this;
    maximum(maximum: number): this;
    minimum(minimum: number): this;
    exclusiveMaximum(exclusiveMaximum: number | boolean): this;
    exclusiveMinimum(exclusiveMinimum: number | boolean): this;
    multipleOf(multipleOf: number): this;
    maxLength(maxLength: number): this;
    minLength(minLength: number): this;
    pattern(pattern: RegExp): this;
    format(format: string): this;
}
declare module './schema' {
    interface Schema<State extends SchemaState> {
        new (...args: any[]): never;
    }
}
